@@ -1,10 +1,30 @@
+# frozen_string_literal: false
 require 'test/unit'
 require 'delegate'
 require 'timeout'
 require 'bigdecimal'
-require_relative 'envutil'
 
 class TestRange < Test::Unit::TestCase
+  def test_new
+    assert_equal((0..2), Range.new(0, 2))
+    assert_equal((0..2), Range.new(0, 2, false))
+    assert_equal((0...2), Range.new(0, 2, true))
+
+    assert_raise(ArgumentError) { (1.."3") }
+
+    obj = Object.new
+    def obj.<=>(other)
+      raise RuntimeError, "cmp"
+    end
+    assert_raise_with_message(RuntimeError, "cmp") { (obj..3) }
+  end
+
+  def test_frozen_initialize
+    r = Range.allocate
+    r.freeze
+    assert_raise(FrozenError){r.__send__(:initialize, 1, 2)}
+  end
+
   def test_range_string
     # XXX: Is this really the test of Range?
     assert_equal([], ("a" ... "a").to_a)
@@ -60,6 +80,9 @@ class TestRange < Test::Unit::TestCase
 
     assert_equal(0, (0..0).min)
     assert_equal(nil, (0...0).min)
+
+    assert_equal([0,1,2], (0..10).min(3))
+    assert_equal([0,1], (0..1).min(3))
   end
 
   def test_max
@@ -77,6 +100,9 @@ class TestRange < Test::Unit::TestCase
 
     assert_equal(0, (0..0).max)
     assert_equal(nil, (0...0).max)
+
+    assert_equal([10,9,8], (0..10).max(3))
+    assert_equal([9,8,7], (0...10).max(3))
   end
 
   def test_initialize_twice
@@ -92,6 +118,16 @@ class TestRange < Test::Unit::TestCase
     assert_nothing_raised { r.instance_eval { initialize 5, 6} }
   end
 
+  def test_marshal
+    r = 1..2
+    assert_equal(r, Marshal.load(Marshal.dump(r)))
+    r = 1...2
+    assert_equal(r, Marshal.load(Marshal.dump(r)))
+    s = Marshal.dump(r)
+    s.sub!(/endi./n, 'end0')
+    assert_raise(ArgumentError) {Marshal.load(s)}
+  end
+
   def test_bad_value
     assert_raise(ArgumentError) { (1 .. :a) }
   end
@@ -126,7 +162,10 @@ class TestRange < Test::Unit::TestCase
   end
 
   def test_hash
-    assert_kind_of(Fixnum, (0..1).hash)
+    assert_kind_of(Integer, (0..1).hash)
+    assert_equal((0..1).hash, (0..1).hash)
+    assert_not_equal((0..1).hash, (0...1).hash)
+    assert_kind_of(String, (0..1).hash.to_s)
   end
 
   def test_step
@@ -246,6 +285,7 @@ class TestRange < Test::Unit::TestCase
   def test_begin_end
     assert_equal(0, (0..1).begin)
     assert_equal(1, (0..1).end)
+    assert_equal(1, (0...1).end)
   end
 
   def test_first_last
@@ -260,7 +300,10 @@ class TestRange < Test::Unit::TestCase
     assert_equal([0, 1, 2], (0...10).first(3))
     assert_equal([7, 8, 9], (0...10).last(3))
     assert_equal(0, (0...10).first)
+    assert_equal(10, (0...10).last)
     assert_equal("a", ("a"..."c").first)
+    assert_equal("c", ("a"..."c").last)
+    assert_equal(0, (2...0).last)
   end
 
   def test_to_s
@@ -296,6 +339,30 @@ class TestRange < Test::Unit::TestCase
     }
   end
 
+  def test_eqq_non_linear
+    bug12003 = '[ruby-core:72908] [Bug #12003]'
+    c = Class.new {
+      attr_reader :value
+
+      def initialize(value)
+        @value = value
+      end
+
+      def succ
+        self.class.new(@value.succ)
+      end
+
+      def ==(other)
+        @value == other.value
+      end
+
+      def <=>(other)
+        @value <=> other.value
+      end
+    }
+    assert_operator(c.new(0)..c.new(10), :===, c.new(5), bug12003)
+  end
+
   def test_include
     assert_include("a".."z", "c")
     assert_not_include("a".."z", "5")
@@ -321,12 +388,15 @@ class TestRange < Test::Unit::TestCase
     assert_raise(TypeError) { [][o] }
     class << o; attr_accessor :end end
     o.end = 0
-    assert_raise(NoMethodError) { [][o] }
+    assert_raise(TypeError) { [][o] }
     def o.exclude_end=(v) @exclude_end = v end
     def o.exclude_end?() @exclude_end end
     o.exclude_end = false
     assert_nil([0][o])
     assert_raise(RangeError) { [0][o] = 1 }
+    class << o
+      private :begin, :end
+    end
     o.begin = 10
     o.end = 10
     assert_nil([0][o])
@@ -391,6 +461,10 @@ class TestRange < Test::Unit::TestCase
     assert_raise(TypeError) do
       (1..42).bsearch{ "not ok" }
     end
+    c = eval("class C\u{309a 26a1 26c4 1f300};self;end")
+    assert_raise_with_message(TypeError, /C\u{309a 26a1 26c4 1f300}/) do
+      (1..42).bsearch {c.new}
+    end
     assert_equal (1..42).bsearch{}, (1..42).bsearch{false}
   end
 
@@ -475,20 +549,22 @@ class TestRange < Test::Unit::TestCase
     assert_in_delta(7.0, (0.0..10).bsearch {|x| 7.0 - x })
   end
 
-  def check_bsearch_values(range, search)
+  def check_bsearch_values(range, search, a)
     from, to = range.begin, range.end
     cmp = range.exclude_end? ? :< : :<=
+    r = nil
 
-    # (0) trivial test
-    r = Range.new(to, from, range.exclude_end?).bsearch do |x|
-      fail "#{to}, #{from}, #{range.exclude_end?}, #{x}"
-    end
-    assert_equal nil, r
+    a.for "(0) trivial test" do
+      r = Range.new(to, from, range.exclude_end?).bsearch do |x|
+        fail "#{to}, #{from}, #{range.exclude_end?}, #{x}"
+      end
+      assert_nil r
 
-    r = (to...to).bsearch do
-      fail
+      r = (to...to).bsearch do
+        fail
+      end
+      assert_nil r
     end
-    assert_equal nil, r
 
     # prepare for others
     yielded = []
@@ -497,46 +573,53 @@ class TestRange < Test::Unit::TestCase
       val >= search
     end
 
-    # (1) log test
-    max = case from
-          when Float then 65
-          when Integer then Math.log(to-from+(range.exclude_end? ? 0 : 1), 2).to_i + 1
-          end
-    assert_operator yielded.size, :<=, max
-
-    # (2) coverage test
-    expect =  if search < from
-                from
-              elsif search.send(cmp, to)
-                search
-              else
-                nil
-              end
-    assert_equal expect, r
-
-    # (3) uniqueness test
-    assert_equal nil, yielded.uniq!
-
-    # (4) end of range test
-    case
-    when range.exclude_end?
-      assert_not_include yielded, to
-      assert_not_equal r, to
-    when search >= to
-      assert_include yielded, to
-      assert_equal search == to ? to : nil, r
+    a.for "(1) log test" do
+      max = case from
+            when Float then 65
+            when Integer then Math.log(to-from+(range.exclude_end? ? 0 : 1), 2).to_i + 1
+            end
+      assert_operator yielded.size, :<=, max
     end
 
-    # start of range test
-    if search <= from
-      assert_include yielded, from
-      assert_equal from, r
+    a.for "(2) coverage test" do
+      expect = case
+               when search < from
+                 from
+               when search.send(cmp, to)
+                 search
+               else
+                 nil
+               end
+      assert_equal expect, r
     end
 
-    # (5) out of range test
-    yielded.each do |val|
-      assert_operator from, :<=, val
-      assert_send [val, cmp, to]
+    a.for "(3) uniqueness test" do
+      assert_nil yielded.uniq!
+    end
+
+    a.for "(4) end of range test" do
+      case
+      when range.exclude_end?
+        assert_not_include yielded, to
+        assert_not_equal r, to
+      when search >= to
+        assert_include yielded, to
+        assert_equal search == to ? to : nil, r
+      end
+    end
+
+    a.for "(5) start of range test" do
+      if search <= from
+        assert_include yielded, from
+        assert_equal from, r
+      end
+    end
+
+    a.for "(6) out of range test" do
+      yielded.each do |val|
+        assert_operator from, :<=, val
+        assert_send [val, cmp, to]
+      end
     end
   end
 
@@ -544,10 +627,12 @@ class TestRange < Test::Unit::TestCase
     ints   = [-1 << 100, -123456789, -42, -1, 0, 1, 42, 123456789, 1 << 100]
     floats = [-Float::INFINITY, -Float::MAX, -42.0, -4.2, -Float::EPSILON, -Float::MIN, 0.0, Float::MIN, Float::EPSILON, Math::PI, 4.2, 42.0, Float::MAX, Float::INFINITY]
 
-    [ints, floats].each do |values|
-      values.combination(2).to_a.product(values).each do |(from, to), search|
-        check_bsearch_values(from..to, search)
-        check_bsearch_values(from...to, search)
+    all_assertions do |a|
+      [ints, floats].each do |values|
+        values.combination(2).to_a.product(values).each do |(from, to), search|
+          check_bsearch_values(from..to, search, a)
+          check_bsearch_values(from...to, search, a)
+        end
       end
     end
   end
@@ -567,17 +652,6 @@ class TestRange < Test::Unit::TestCase
     assert_raise(TypeError) { ("a".."z").bsearch {} }
   end
 
-  def test_bsearch_with_mathn
-    assert_separately ['-r', 'mathn'], %q{
-      msg = '[ruby-core:25740]'
-      answer = (1..(1 << 100)).bsearch{|x|
-        assert_predicate(x, :integer?, msg)
-        x >= 42
-      }
-      assert_equal(42, answer, msg)
-    }
-  end
-
   def test_each_no_blockarg
     a = "a"
     def a.upto(x, e, &b)
