@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+# frozen_string_literal: true
 #
 # test/strscan/test_stringscanner.rb
 #
@@ -12,7 +14,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal false, s.eos?
     assert_equal false, s.tainted?
 
-    str = 'test string'
+    str = 'test string'.dup
     str.taint
     s = StringScanner.new(str, false)
     assert_instance_of StringScanner, s
@@ -20,7 +22,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_same str, s.string
     assert_equal true, s.string.tainted?
 
-    str = 'test string'
+    str = 'test string'.dup
     str.taint
     s = StringScanner.new(str)
     assert_equal true, s.string.tainted?
@@ -31,8 +33,8 @@ class TestStringScanner < Test::Unit::TestCase
   def test_s_allocate
     s = StringScanner.allocate
     assert_equal '#<StringScanner (uninitialized)>', s.inspect.sub(/StringScanner_C/, 'StringScanner')
-    assert_raises(UNINIT_ERROR) { s.eos? }
-    assert_raises(UNINIT_ERROR) { s.scan(/a/) }
+    assert_raise(UNINIT_ERROR) { s.eos? }
+    assert_raise(UNINIT_ERROR) { s.scan(/a/) }
     s.string = 'test'
     assert_equal '#<StringScanner 0/4 @ "test">', s.inspect.sub(/StringScanner_C/, 'StringScanner')
     assert_nothing_raised(UNINIT_ERROR) { s.eos? }
@@ -94,7 +96,7 @@ class TestStringScanner < Test::Unit::TestCase
   end
 
   def test_inspect
-    str = 'test string'
+    str = 'test string'.dup
     str.taint
     s = StringScanner.new(str, false)
     assert_instance_of String, s.inspect
@@ -122,7 +124,7 @@ class TestStringScanner < Test::Unit::TestCase
     s.scan(/\w+/)
     assert_equal true, s.eos?
 
-    s = StringScanner.new('test')
+    s = StringScanner.new('test'.dup)
     s.scan(/te/)
     s.string.replace ''
     assert_equal true, s.eos?
@@ -168,6 +170,26 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal 0, s.pos
   end
 
+  def test_string_set_is_equal
+    name = 'tenderlove'
+
+    s = StringScanner.new(name)
+    assert_equal name.object_id, s.string.object_id
+
+    s.string = name
+    assert_equal name.object_id, s.string.object_id
+  end
+
+  def test_string_append
+    s = StringScanner.new('tender'.dup)
+    s << 'love'
+    assert_equal 'tenderlove', s.string
+
+    s.string = 'tender'.dup
+    s << 'love'
+    assert_equal 'tenderlove', s.string
+  end
+
   def test_pos
     s = StringScanner.new('test string')
     assert_equal 0, s.pos
@@ -179,8 +201,19 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal 11, s.pos
   end
 
+  def test_pos_unicode
+    s = StringScanner.new("abcädeföghi")
+    assert_equal 0, s.charpos
+    assert_equal "abcä", s.scan_until(/ä/)
+    assert_equal 4, s.charpos
+    assert_equal "defö", s.scan_until(/ö/)
+    assert_equal 8, s.charpos
+    s.terminate
+    assert_equal 11, s.charpos
+  end
+
   def test_concat
-    s = StringScanner.new('a')
+    s = StringScanner.new('a'.dup)
     s.scan(/a/)
     s.concat 'b'
     assert_equal false, s.eos?
@@ -213,7 +246,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil           s.scan(/\w+/)
 
 
-    str = 'stra strb strc'
+    str = 'stra strb strc'.dup
     str.taint
     s = StringScanner.new(str, false)
     tmp = s.scan(/\w+/)
@@ -234,7 +267,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil           s.scan(/\w+/)
     assert_nil           s.scan(/\w+/)
 
-    s = StringScanner.new('test')
+    s = StringScanner.new('test'.dup)
     s.scan(/te/)
     # This assumes #string does not duplicate string,
     # but it is implementation specific issue.
@@ -260,7 +293,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil      s.skip(/\s+/)
     assert_equal true, s.eos?
 
-    s = StringScanner.new('test')
+    s = StringScanner.new('test'.dup)
     s.scan(/te/)
     s.string.replace ''
     assert_equal nil, s.skip(/./)
@@ -280,7 +313,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal 'e', s.getch
     assert_nil        s.getch
 
-    str = 'abc'
+    str = 'abc'.dup
     str.taint
     s = StringScanner.new(str)
     assert_equal true, s.getch.tainted?
@@ -288,11 +321,11 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal true, s.getch.tainted?
     assert_nil s.getch
 
-    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
-    assert_equal "\244\242".force_encoding("euc-jp"), s.getch
+    s = StringScanner.new("\244\242".dup.force_encoding("euc-jp"))
+    assert_equal "\244\242".dup.force_encoding("euc-jp"), s.getch
     assert_nil s.getch
 
-    s = StringScanner.new('test')
+    s = StringScanner.new('test'.dup)
     s.scan(/te/)
     s.string.replace ''
     assert_equal nil, s.getch
@@ -308,7 +341,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil        s.get_byte
     assert_nil        s.get_byte
 
-    str = 'abc'
+    str = 'abc'.dup
     str.taint
     s = StringScanner.new(str)
     assert_equal true, s.get_byte.tainted?
@@ -316,12 +349,12 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal true, s.get_byte.tainted?
     assert_nil s.get_byte
 
-    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
-    assert_equal "\244".force_encoding("euc-jp"), s.get_byte
-    assert_equal "\242".force_encoding("euc-jp"), s.get_byte
+    s = StringScanner.new("\244\242".dup.force_encoding("euc-jp"))
+    assert_equal "\244".dup.force_encoding("euc-jp"), s.get_byte
+    assert_equal "\242".dup.force_encoding("euc-jp"), s.get_byte
     assert_nil s.get_byte
 
-    s = StringScanner.new('test')
+    s = StringScanner.new('test'.dup)
     s.scan(/te/)
     s.string.replace ''
     assert_equal nil, s.get_byte
@@ -354,7 +387,7 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal 't', s.matched
     assert_equal false, s.matched.tainted?
 
-    str = 'test'
+    str = 'test'.dup
     str.taint
     s = StringScanner.new(str)
     s.scan(/\w+/)
@@ -370,6 +403,8 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal 'stra', s[-1]
     assert_equal 'stra', s[0]
     assert_nil           s[1]
+    assert_raise(IndexError) { s[:c] }
+    assert_raise(IndexError) { s['c'] }
 
     assert_equal false,  s[-1].tainted?
     assert_equal false,  s[0].tainted?
@@ -412,11 +447,11 @@ class TestStringScanner < Test::Unit::TestCase
     assert_nil           s[0]
 
 
-    s = StringScanner.new("\244\242".force_encoding("euc-jp"))
+    s = StringScanner.new("\244\242".dup.force_encoding("euc-jp"))
     s.getch
-    assert_equal "\244\242".force_encoding("euc-jp"), s[0]
+    assert_equal "\244\242".dup.force_encoding("euc-jp"), s[0]
 
-    str = 'test'
+    str = 'test'.dup
     str.taint
     s = StringScanner.new(str)
     s.scan(/(t)(e)(s)(t)/)
@@ -425,6 +460,19 @@ class TestStringScanner < Test::Unit::TestCase
     assert_equal true, s[2].tainted?
     assert_equal true, s[3].tainted?
     assert_equal true, s[4].tainted?
+
+    s = StringScanner.new("foo bar baz")
+    s.scan(/(?<a>\w+) (?<b>\w+) (\w+)/)
+    assert_equal 'foo', s[1]
+    assert_equal 'bar', s[2]
+    assert_nil s[3]
+    assert_equal 'foo', s[:a]
+    assert_equal 'bar', s[:b]
+    assert_raise(IndexError) { s[:c] }
+    assert_equal 'foo', s['a']
+    assert_equal 'bar', s['b']
+    assert_raise(IndexError) { s['c'] }
+    assert_raise_with_message(IndexError, /\u{30c6 30b9 30c8}/) { s["\u{30c6 30b9 30c8}"] }
   end
 
   def test_pre_match
@@ -448,7 +496,7 @@ class TestStringScanner < Test::Unit::TestCase
     s.scan(/never match/)
     assert_nil s.pre_match
 
-    str = 'test string'
+    str = 'test string'.dup
     str.taint
     s = StringScanner.new(str)
     s.scan(/\w+/)
@@ -482,7 +530,7 @@ class TestStringScanner < Test::Unit::TestCase
     s.scan(/./)
     assert_nil s.post_match
 
-    str = 'test string'
+    str = 'test string'.dup
     str.taint
     s = StringScanner.new(str)
     s.scan(/\w+/)
@@ -528,9 +576,8 @@ class TestStringScanner < Test::Unit::TestCase
     s.terminate
     assert_nil s.matched_size
 
-    # obsolete
     s = StringScanner.new('test string')
-    assert_nil s.matchedsize
+    assert_nil s.matched_size
     s.scan(/test/)
     assert_equal 4, s.matched_size
     s.terminate
@@ -538,14 +585,14 @@ class TestStringScanner < Test::Unit::TestCase
   end
 
   def test_encoding
-    ss = StringScanner.new("\xA1\xA2".force_encoding("euc-jp"))
+    ss = StringScanner.new("\xA1\xA2".dup.force_encoding("euc-jp"))
     assert_equal(Encoding::EUC_JP, ss.scan(/./e).encoding)
   end
 
   def test_generic_regexp
-    ss = StringScanner.new("\xA1\xA2".force_encoding("euc-jp"))
+    ss = StringScanner.new("\xA1\xA2".dup.force_encoding("euc-jp"))
     t = ss.scan(/./)
-    assert_equal("\xa1\xa2".force_encoding("euc-jp"), t)
+    assert_equal("\xa1\xa2".dup.force_encoding("euc-jp"), t)
   end
 
   def test_set_pos
@@ -666,9 +713,41 @@ class TestStringScanner < Test::Unit::TestCase
     s.scan(/ string/)
   end
 
-  def test_inspect
+  def test_inspect2
     s = StringScanner.new('test string test')
     s.scan(/test strin/)
     assert_equal('#<StringScanner 10/16 "...strin" @ "g tes...">', s.inspect)
   end
+
+  def test_aref_without_regex
+    s = StringScanner.new('abc')
+    s.get_byte
+    assert_nil(s[:c])
+    assert_nil(s["c"])
+    s.getch
+    assert_nil(s[:c])
+    assert_nil(s["c"])
+  end
+
+  def test_size
+    s = StringScanner.new("Fri Dec 12 1975 14:39")
+    s.scan(/(\w+) (\w+) (\d+) /)
+    assert_equal(4, s.size)
+  end
+
+  def test_captures
+    s = StringScanner.new("Fri Dec 12 1975 14:39")
+    s.scan(/(\w+) (\w+) (\d+) /)
+    assert_equal(["Fri", "Dec", "12"], s.captures)
+    s.scan(/(\w+) (\w+) (\d+) /)
+    assert_nil(s.captures)
+  end
+
+  def test_values_at
+    s = StringScanner.new("Fri Dec 12 1975 14:39")
+    s.scan(/(\w+) (\w+) (\d+) /)
+    assert_equal(["Fri Dec 12 ", "12", nil, "Dec"], s.values_at(0, -1, 5, 2))
+    s.scan(/(\w+) (\w+) (\d+) /)
+    assert_nil(s.values_at(0, -1, 5, 2))
+  end
 end
